#ifndef TASKS_H
#define TASKS_H

#ifndef SWIG
#include <gmpxx.h>
#endif

class Task
{
  private:
    unsigned long period;
    unsigned long wcet;
    unsigned long deadline;

  public:

    /* construction and initialization */
    void init(unsigned long wcet, unsigned long period, unsigned long deadline = 0);
    Task(unsigned long wcet = 0,
	 unsigned long period = 0,
	 unsigned long deadline = 0) { init(wcet, period, deadline); }

    /* getter / setter */
    unsigned long get_period() const { return period;   }
    unsigned long get_wcet() const   { return wcet;     }
    /* defaults to implicit deadline */
    unsigned long get_deadline() const
    {
        return deadline != 0 ? deadline : period;
    }

    void set_period(unsigned long period)     { this->period   = period;   }
    void set_wcet(unsigned long wcet)         { this->wcet     = wcet;     }
    void set_deadline(unsigned long deadline) { this->deadline = deadline; }

    /* properties */
    bool has_implicit_deadline() const;
    bool has_constrained_deadline() const;
    bool is_feasible() const;


    void get_utilization(mpq_class &util) const;
    void get_density(mpq_class &density) const;
};

class TaskSet
{
  private:
    unsigned int num_tasks;
    Task*        tasks;

    void allocate();

  public:
    TaskSet(unsigned int num_tasks);
    TaskSet(const TaskSet &original);
    virtual ~TaskSet();

    unsigned int get_task_count() const { return num_tasks; }

    Task& operator[](int idx) { return tasks[idx]; }

    const Task& operator[](int idx) const { return tasks[idx]; }

    int  get_index(Task& tsk) const { return &tsk - tasks; }

    bool has_only_implicit_deadlines() const;
    bool has_only_constrained_deadlines() const;
    bool has_only_feasible_tasks() const;
    bool is_not_overutilized(unsigned int num_processors) const;

    void get_utilization(mpq_class &util) const;
    void get_density(mpq_class &density) const;
    void get_max_density(mpq_class &max_density) const;

    /* wrapper for Python access */
    unsigned long get_period(unsigned int idx) const
    {
        return tasks[idx].get_period();
    }

    unsigned long get_wcet(unsigned int idx) const
    {
        return tasks[idx].get_wcet();
    }

    unsigned long get_deadline(unsigned int idx) const
    {
        return tasks[idx].get_deadline();
    }

    void set_period(unsigned int idx, unsigned long x)
    {
        tasks[idx].set_period(x);
    }

    void set_wcet(unsigned int idx, unsigned long x)
    {
        tasks[idx].set_wcet(x);
    }

    void set_deadline(unsigned int idx, unsigned long x)
    {
        tasks[idx].set_deadline(x);
    }
};



#endif
