#!/usr/bin/env python

import gedf
import model import serialize
import xml.etree.ElementTree as ET

from util import maybe_int
from util.quantor import forall

import optparse
import sys

options = None

def annotate_gedf(ts, no_cpus):

    if no_cpus >= ts.utilization() and \
            forall(ts)(lambda t: t.density() <= 1.0):
        gfb  = gedf.gfb.is_schedulable(no_cpus, ts)
        bcl  = gedf.bcl.is_schedulable(no_cpus, ts)
        bcli = gedf.bcli.is_schedulable(no_cpus, ts)
        skb  = gedf.bar.is_schedulable(no_cpus, ts)
        bak  = gedf.bak.is_schedulable(no_cpus, ts)
    else:
        gfb  = False
        bcl  = False
        bcli = False
        skb  = False
        bak  = False

    found = None
    for node in ts.xml.findall('schedulable'):
        m = maybe_int(node.get('cpus', None))
        if m == no_cpus:
            found = node
            break

    if found is None:
        found = ET.SubElement(ts.xml, 'schedulable')
        found.set('cpus', str(no_cpus))

    updates = [('gedf', 'SKB', skb),
               ('gedf', 'GFB', gfb),
               ('gedf', 'BCL', bcl),
               ('gedf', 'BCLI', bcli),
               ('gedf', 'BAK', bak)]

    for (name, test, res) in updates:
        # clean out old annotations, if any
        for x in found.findall(name):
            if x.get('test', None) == test:
                found.remove(x)
        ET.SubElement(found, name, test=str(test), result=str(res))


def annotate_gedf_sched(ts):
    for m in options.cpus:
        annotate_gedf(ts, m)
    return ts

def drop_sched(ts):
    for x in ts.xml.findall('schedulable'):
        ts.xml.remove(x)
    return ts

def convert_to_us(ts):
    ts.make_integral_us()
    ts.xml = serialize.taskset(ts) # remove old XML
    return ts

def annotate_file(cpus, file, updates=[]):
    ts   = serialize.load(file)
    for f in updates:
        ts = f(ts)
    serialize.write(ts.xml, file)

o = optparse.make_option
opts = [
    o('-m', '--cpus', action='append', dest='cpus', type='int',
      help='Number of CPUs to test.'),
]

defaults = {
    'cpus' : []
}

updates = {
    'gedf'           : annotate_gedf_sched,
    'drop-sched'     : drop_sched,
    'to-us'          : convert_to_us,
}

def main():
    global options
    parser = optparse.OptionParser(option_list=opts)
    parser.set_defaults(**defaults)
    (options, args) = parser.parse_args()

    ops = []

    while args and args[0] in updates:
        ops += [updates[args[0]]]
        del args[0]

    for file in args:
        print 'Processing %s...' % file
        annotate_file(options.cpus, file, ops)

if __name__ == '__main__':
    main()
