#!/usr/bin/env python

from itertools import cycle
from random import uniform
from model import serialize

import compare_impl as ci
import generator.tasksets as gen
import gedf

from util import Timer


def run_python_vs_cpp(constrained=False, stop_after=None,
                      umax=32.0, cpus=32):

    if not gedf.using_cpp:
        print "C++ not available; skipping test."
        return

    ok  = 0
    bad = 0
    ts_count = 0

    creation = Timer()
    py  = Timer()
    cpp = Timer()
    sim = Timer()

    try:
        for gn in cycle(gen.DIST.keys()):
            g = gen.DIST[gn]

            creation.start()
            ucap = uniform(1.0, umax)
            ts = g(ucap, None)
            if constrained:
                for t in ts:
                    t.deadline = uniform(t.cost, t.period)
            print "testing %.2f / %d task set" % (ucap, len(ts))
            ts.make_integral_us()
            creation.stop()

            py.start()
            py_res = gedf.all.is_schedulable_py(cpus, ts)
            py.stop()

            cpp.start()
            cpp_res = gedf.all.is_schedulable_cpp(cpus, ts)
            cpp.stop()

            sim.start()
            sim_res = gedf.sim.no_counter_example(cpus, ts)
            sim.stop()

            if py_res != cpp_res:
                bad += 1
                print "Task set %d failed comparison: py=%d cpp=%d" % \
                    (ok + bad, py_res, cpp_res)
                fname = "/tmp/failed-task-set-comp-%d.xml" % (bad)
                serialize.write_ts(ts, fname)
                print "Dumped task set to %s." % fname
            elif sim_res > py_res:
                bad += 1
                print "Task set %d failed bound: test=%d sim=%d" % \
                    (ok + bad, py_res, sim_res)
                fname = "/tmp/failed-task-set-lower-%d.xml" % (bad)
                serialize.write_ts(ts, fname)
                print "Dumped task set to %s." % fname
            else:
                ok += 1
            ts_count += 1
            if stop_after != None and ts_count >= stop_after:
                break
    except KeyboardInterrupt:
        pass
    print "Test aborted; %d ok and %d bad, tested %d tasksets." \
        % (ok, bad, ts_count)
    print "%17s%17s%17s%17s%17s" % ("Python", "C++", "Ratio", "Creation", "Simulation")
    print "%17s%17s%17.2f%17s%17s" \
            % (py.elapsed, cpp.elapsed,
               py.seconds() / cpp.seconds()
               if cpp.seconds() != 0 else 0,
               creation.elapsed, sim.elapsed)


if __name__ == '__main__':
    count = 10
    print "Implicit:"
    run_python_vs_cpp(stop_after=count, umax=31.5)
    print "Constrained:"
    run_python_vs_cpp(stop_after=count, umax=20,
                      constrained=True)
    print "Re-test until schedulable:"
    ci.run_test_loop(stop_after=count, umax=12.0)
