import sys
import os

from random import uniform
from functools import partial
from fractions import Fraction
from datetime import datetime

import generator.tasksets as gen
import gedf
from compare_impl import run_py_test, MAX_CPUS, abort_on_sigusr1
from util import Counter


def testsets(ucap=10.0):
    for gn in gen.DIST:
        ugen = gen.DIST[gn]
        for dgn in gen.CONSTRAINED_DEADLINE_DIST:
            dlgen = gen.CONSTRAINED_DEADLINE_DIST[dgn]
            umax = uniform(1, ucap)
            ts = ugen(umax, None)
            dlgen(ts)
            ts.make_integral_us()
            ts.ugen  = gn
            ts.dlgen = dgn
            yield ts

def search_ffdbf():
    num = Counter()
    for ts in testsets(ucap=10.0):
        num.inc()
        print '+++++ %s:%s u:%.2f d:%.2f md:%2f at %s (%d) [pid:%d]' % \
            (ts.ugen, ts.dlgen, ts.utilization(),
             ts.density(), ts.max_density(), datetime.now(), num.val,
             os.getpid())
        res = run_py_test(ts, partial(gedf.ffdbf.is_schedulable, epsilon=Fraction(1, 10)), max_cpus=MAX_CPUS)
        print '----- res =', res
        sys.stdout.flush()

if __name__ == '__main__':
    abort_on_sigusr1()
    search_ffdbf()
    print "Terminating."
