import random
import sample

from overheads import Overheads

# reuse experimental setup, but use real overheads
from jsa10 import gedf_no_overheads_test, gedf_proc_or_task_centric_irq_test, gedf_quantum_centric_irq_test

# RTSS'09
def run(generator, soft,
        umin, umax, epsilon,
        cpus, samples, min_delta=0.01,
        points=None,
        **kargs):
    gsn = Overheads()
    gfg = Overheads()
    gq  = Overheads()
    sgq = Overheads()
    ghq = Overheads()
    gsn_rm = Overheads()
    gfg_rm = Overheads()
    gq_rm  = Overheads()
    sgq_rm = Overheads()

    if soft:
        gsn.gsnedf_soft()
        gfg.gfgedf_soft()
        gq.gqedf_soft()
        ghq.ghqedf_soft()
        gsn_rm.gsnedf_rm_soft()
        gfg_rm.gfgedf_rm_soft()
        gq_rm.gqedf_rm_soft()
    else:
        gsn.gsnedf_hard()
        gfg.gfgedf_hard()
        gq.gqedf_hard()
        sgq.sgqedf_hard()
        ghq.ghqedf_hard()
        gsn_rm.gsnedf_rm_hard()
        gfg_rm.gfgedf_rm_hard()
        gq_rm.gqedf_rm_hard()
        sgq_rm.sgqedf_rm_hard()

    tests = [
        gedf_no_overheads_test(cpus, soft),
        gedf_proc_or_task_centric_irq_test(cpus, soft, gsn),
        gedf_proc_or_task_centric_irq_test(cpus - 1, soft, gsn_rm, release_master=True),
        gedf_proc_or_task_centric_irq_test(cpus, soft, gfg),
        gedf_proc_or_task_centric_irq_test(cpus - 1, soft, gfg_rm, release_master=True),
        gedf_proc_or_task_centric_irq_test(cpus, soft, ghq),
        gedf_quantum_centric_irq_test(cpus, soft, gq),
        gedf_quantum_centric_irq_test(cpus - 1, soft, gq_rm, release_master=True),
        ]

    staggered_hard = [
        gedf_quantum_centric_irq_test(cpus, soft, sgq, stagger=True),
        gedf_quantum_centric_irq_test(cpus - 1, soft, sgq_rm, stagger=True, release_master=True),
        ]

    if not soft:
        tests += staggered_hard

    if not points:
        ret = sample.range_adaptive([umin], [umax],
                                    sample.close_enough_epsilon(epsilon, min_delta=min_delta),
                                    sample.middle_point_scalar,
                                    lambda u: lambda: [generator, u,
                                                       random.getstate()],
                                    tests, samples=samples, **kargs)
    else:
        ret = sample.range(points,
                           lambda u: lambda: [generator, u,
                                              random.getstate()],
                           tests, samples=samples, **kargs)
    ret.sort(key=lambda x: x[0])
    return ret

