from util import us2ms

class ServiceFunction(object):
    def __init__(self, delay=0.0, rate=1.0):
        self.delay = delay  # \sigma_k in paper
        self.rate  = rate   # \hat{u}_k in paper

    def __str__(self):
        return "max(0, %s * (delta - %s))" % (self.rate, self.delay)

    def __call__(self, delta):
        return max(0, self.rate * (delta - self.delay))

    def is_restricted(self):
        return self.rate < 1 or self.delay > 0

    @staticmethod
    def from_overheads(overheads, tasks, release_master=False):
        n          = len(tasks)
        rel_charge = us2ms(overheads.release(tasks))
        delay      = n * rel_charge + us2ms(overheads.tick(tasks))
        rate       = 1.0
        # take out 
        rate      -= overheads.tick(tasks) / overheads.quantum_size
        if not release_master:
            for t in tasks:
                # utilization loss due to releases
                rate -= rel_charge / t.period
        return ServiceFunction(delay, rate)


def has_bounded_tardiness(service_funcs, tasks):
    usum = tasks.utilization()
    ssum = sum([p.rate for p in service_funcs])
    # service exceeded?
    if usum > ssum:
        return False
    # denominator of Theorem 3 positive?
    # rcount = H in the paper
    rcount = sum([p.is_restricted() for p in service_funcs])
    # umax = max(u_l) in the paper
    umax   = tasks.max_utilization()
    m = len(service_funcs)
    # heavy_sum = U_L  in the paper
    heavy_sum = tasks.utilization(heaviest=m - 1)
    return ssum > max(rcount - 1, 0) * umax + heavy_sum
