#!/usr/bin/env python

from subprocess import call
from tempfile import NamedTemporaryFile
from re import match

SOLUTION_UNDEFINED  = 1
SOLUTION_FEASIBLE   = 2
SOLUTION_INFEASIBLE = 3
SOLUTION_NOT_EXISTS = 4
SOLUTION_OPTIMAL    = 5
SOLUTION_UNBOUNDED  = 6

class GlpSolResult(object):
    def __init__(self, prog, solution, hsolution, report):
        self.prog = prog
        self.raw  = solution
        self.formatted = "".join(hsolution)
        self.log  = "".join(report)
        self.line = {}
        self.val  = {}
        dim = solution[0] # first line has dimensions

        m = match("(\d+) (\d+)", dim)
        self.rows = int(m.group(1))
        self.cols = int(m.group(2))

        if prog.has_integer_vars():
            self.read_mip()
        else:
            self.read_lp()

    def read_mip(self):
        res = self.raw[1] # second line has objective
        m = match("(\d+) (.+)", res)
        self.status    = int(m.group(1))
        self.objective = float(m.group(2))

# Broken: assumes wrong output order.
#        for (obj, line) in zip(self.prog.c_ord + self.prog.v_ord, self.raw[2:]):
#            self.line[obj] = line
#            self.val[obj] = float(line)

    def read_lp(self):
        res = self.raw[1] # second line has objective
        m = match("(\d+) ([^ ]+) ([^ ]+)", res)
        self.status    = int(m.group(1))
        self.objective = float(m.group(2))
        self.dual_obj  = float(m.group(3))

        self.dual_val = {}

# Broken: assumes wrong output order.
#        for (obj, line) in zip(self.prog.c_ord + self.prog.v_ord, self.raw[2:]):
#            self.line[obj] = line
#            m = match("(\d+) ([^ ]+) ([^ ]+)", line)
#            self.val[obj] = float(m.group(2))
#            self.dual_val[obj] = float(m.group(3))


def glpsol(linprog):
    # create input file
    model  = NamedTemporaryFile()
    model.write(linprog.format_lp())
    model.flush()

    # create output file
    result = NamedTemporaryFile()

    # create human-readable file
    hresult = NamedTemporaryFile()

    # create log file
    log    = NamedTemporaryFile()


    args = ['glpsol',
            '--cpxlp', model.name,
            '-w', result.name,
            '-o', hresult.name,
            '--log', log.name]

    null = open('/dev/null', 'w')
    ret = call(args, stdout=null)
    null.close()

    print "ret", ret

    f = open(log.name)
    report   = f.readlines()
    f.close()

    f = open(result.name)
    raw_solution = f.readlines()
    f.close()

    f = open(hresult.name)
    human_solution = f.readlines()
    f.close()

    del model
    del log
    del result
    del hresult

    return GlpSolResult(linprog, raw_solution, human_solution, report)
