from itertools import chain

class Resource(object):
    def __init__(self, name, group=None):
        self.name     = name
        self.requests = []
        if group:
            group.add_resources([self])
        else:
            ResGroup(self)

    def __str__(self):
        return "Resource(%s)" % str(self.name)

    def Request(self, task, duration, *args, **kargs):
        return Request(task, self, duration, *args, **kargs)

    def add_request(self, req):
        # stick into res group
        self.requests += [req]
        # tell group to recompute
        self.group.drop_caches()
        # merge groups if required by nesting
        if req.outer:
            ResGroup.merge(self.group, req.outer.resource.group)

    def remove_all_nested_requests(self):
        self.requests = [req for req in self.requests if req.outer is None]
        self.group.drop_caches()


class Request(object):
    def __init__(self, task, resource, duration, outer=None, type=None, period=1):
        self.task         = task     # Who needs it?
        self.type         = type     # Anything special about this request?
        self.inner        = []       # List of nested inner requests.
        self.outer        = outer    # Default: no nesting.
        self.duration     = duration # Cumulative duration.
        self.raw_duration = duration # Duration without nested requests.
        self.resource     = resource
        self.period       = period
        if outer:
            outer.add_nested(self)
        resource.add_request(self)

    def __str__(self):
        tstr = ", type=%s" % self.type if not self.type is None else ""
        pstr = ", period=%s" % self.period if self.period != 1 else ""
        return "Request(%s, %s, %s%s%s)" % (self.task, self.resource,
                                            self.duration, tstr, pstr)

    def add_nested(self, nested):
        self.inner   += [nested]
        r = self
        while r:
            r.duration += nested.duration
            r = r.outer

    def Nested(self, resource, duration, **kargs):
        return Request(self.task, resource, duration, outer=self, **kargs)

class ResGroup(object):
    def __init__(self, *initial_members):
        self.resources = []   # resources in this group
        self.__reqs    = None # cached requests
        self.__tsks    = None # cached tasks
        self.add_resources(initial_members)

    def drop_caches(self, *args):
        self.__reqs     = None
        self.__tsks     = None

    def add_resources(self, resources):
        self.drop_caches()
        self.resources += resources
        for res in resources:
            res.group = self

    def get_requests(self):
        if self.__reqs is None:
            self.__reqs = []
            for r in self.resources:
                self.__reqs += r.requests
            self.__reqs.sort(key=lambda x: x.duration, reverse=True)
        return self.__reqs
    requests = property(get_requests)

    def get_tasks(self):
        if self.__tsks is None:
            self.__tsks = frozenset([req.task for req in self.requests])
        return self.__tsks
    tasks = property(get_tasks)

    def __str__(self):
        return "ResGroup(%s)" % ", ".join([str(r) for r in self.resources])

    @staticmethod
    def merge(a, b):
        if a != b:
            if len(a.resources) < len(b.resources):
                # merge shorter into longer
                a, b = b, a
            a.add_resources(b.resources)
            b.resources = None
