#!/usr/bin/env python

from model.tasks import TaskSystem, Task
from util import maybe_int
import xml.etree.ElementTree as ET

def task(t):
    tag = ET.Element('task')
    if not t.id is None:
        tag.set('id', str(t.id))
    tag.set('period', str(maybe_int(t.period)))
    tag.set('wcet', str(maybe_int(t.cost)))
    if not t.implicit_deadline():
        tag.set('deadline', str(maybe_int(t.deadline)))
    if t.phase != 0:
        tag.set('phase', str(maybe_int(t.phase)))
    tag.task = t
    task.xml = tag
    return tag

def parse_task(node):
    cost      = float(node.get('wcet'))
    period    = float(node.get('period'))
    kargs = {}
    deadline  = node.get('deadline', None)
    if not deadline is None:
        kargs['deadline'] = float(deadline)
    phase     = node.get('phase', None)
    if not phase is None:
        kargs['phase'] = float(phase)
    id        = node.get('id', None)
    if not id is None:
        kargs['id'] = int(id)

    t = Task(cost, period, **kargs)
    t.xml = node
    node.task = t
    return t

def taskset(ts):
    tag = ET.Element('taskset')

    prop = ET.SubElement(tag, 'properties')
    prop.set('utilization', str(ts.utilization()))
    prop.set('density', str(ts.density()))
    hp = ts.hyperperiod()
    if hp:
        prop.set('hyperperiod', str(hp))

    for t in ts:
        tag.append(task(t))
    return tag

def parse_taskset(node):
    tasks = [parse_task(n) for n in node.findall('task')]
    return TaskSystem(tasks)

def write(root, file):
    tree = ET.ElementTree(root)
    tree.write(file)

def write_ts(ts, file):
    xml = taskset(ts)
    write(xml, file)

def load(file):
    tree = ET.ElementTree()
    tree.parse(file)
    ts = parse_taskset(tree.getroot())
    ts.xml = tree.getroot()
    return ts
