# Do C-EDF assignment also while continuously checking overheads.
# Ugly copy&paste job for now.

from math import ceil, floor

from binpack import DuctileItem, DuctileBin, MaxSpareCapacity, report_failure, DidNotFit

from util.storage import storage
from util.seq     import neighbours

def account_for_timer_tick(exec_cost, timer_period, tick_cost, max_cost):
    num_ticks = int(ceil(exec_cost / float(timer_period)))
    exec_cost += num_ticks * tick_cost
    last_ticks = num_ticks

    num_ticks = int(ceil(exec_cost / float(timer_period)))
    while num_ticks > last_ticks and exec_cost <= max_cost:
        exec_cost += (num_ticks - last_ticks) * tick_cost
        last_ticks = num_ticks
        num_ticks  = int(ceil(exec_cost / float(timer_period)))

    return exec_cost

class InflatedTask(DuctileItem):
    def __init__(self, params, rel_irqs=1, num_tasks=1, density=None):
        self.params = params

        # params.task      --- reference to the task
        # params.cpmd      --- cpmd charge, pre-computed (migratory or preemption)
        # params.needs_ipi --- true if this job may be released on a different CPU

        # this is individual for each task, since periods differ
        self.rel_irqs   = rel_irqs  # number of release interrupts
        self.num_tasks  = num_tasks # number of other tasks
        if density is None:
            self.compute_density()
        else:
            self.density = density

    def num_releases_during(self, delta):
        return int(ceil(delta / float(self.params.task.period)))

    def size(self):
        return self.density

    def copy(self):
        return InflatedTask(self.params,
                            self.rel_irqs,
                            self.num_tasks,
                            self.density)

    def inc_size(self, it):
        self.rel_irqs  += it.num_releases_during(self.params.task.deadline)
        self.num_tasks += 1

    def compute_density(self):
        # compute preempt charge
        tick_cost = self.params.ohead.tick(self.num_tasks)
        preempt  = tick_cost + self.params.cpmd + \
            2 * (self.params.ohead.context_switch(self.num_tasks) +
                 self.params.ohead.sched(self.num_tasks))

        rel_cost = self.params.ohead.release(self.num_tasks)
        # compute inflated utilization
        cost = (self.params.task.cost + preempt +
                self.rel_irqs * rel_cost)

        if self.params.needs_ipi:
            # add in IPI latency
            cost += self.params.ohead.ipi_latency(self.num_tasks)

        cost = account_for_timer_tick(cost, self.params.ohead.tick_period,
                                      tick_cost, self.params.task.period)
        self.inflated_cost = cost
        self.density = cost / float(self.params.task.deadline)

    def update_size(self, it, bin):
        self.inc_size(it)
        self.compute_density()

    def determine_size(self, bin):
        # compute interrupt charge
        self.num_tasks = 1
        self.tm_irqs   = 0
        self.rel_irqs  = 1 # charge for our own delay
        for it in bin.items:
            self.inc_size(it)

        # estimate cost
        self.compute_density()

    def __str__(self):
        return "InflatedTask(%s, %.2f, %.2f)" % (self.params.task,
                                                 self.size(),
                                                 self.size() / self.params.task.density())


def inflated_task(task, ohead, cluster_size, single_cpmd):
    # create non-changing parameters
    params = storage(task=task, cpmd=2 * single_cpmd,
                     needs_ipi = cluster_size > 1,
                     ohead=ohead)
    return InflatedTask(params)


def assign_ts_with_overheads(ts, no_clusters, cluster_size, ohead, max_cpmd, want_bins=False):
    cpus = [DuctileBin(capacity=cluster_size) for _ in range(no_clusters)]

    h = MaxSpareCapacity(initial_bins=cpus)
    # XXX assumes preemptions have the same cost everywhere---true for Ludwig.
    inflated_tasks = [inflated_task(t, ohead, cluster_size, max_cpmd) for t in ts]
    # decreasing density
    inflated_tasks.sort(key = lambda it: it.size(), reverse = True)
    try:
        h.binpack(inflated_tasks, report_misfit=report_failure) # no error reporting
    except DidNotFit:
        return None if want_bins else False

    # great, everything fit

    # for i,b in enumerate(h.bins):
    #     print "Cluster %d" % i
    #     for it in b.items:
    #         print it
    #     sc = b.spare_capacity()
    #     uc = sum([it.params.task.density() for it in b.items])
    #     print 'sc', sc
    #     print 'u', uc, sc + uc
    #     print '=' * 10

    return h.bins if want_bins else True
