import unittest
from fractions import Fraction

import gedf
from model.tasks import Task, TaskSystem


class Test_ffdbf(unittest.TestCase):

    def setUp(self):
        self.t1 = Task(5, 10)
        self.t1.make_integral_us()

        self.t2 = Task(5, 10, deadline = 7)
        self.t2.make_integral_us()

    def test_ffdbf1(self):
        one = Fraction(1)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t1, 0, one),
            0)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t1, 5000, one),
            0)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t1, 5001, one),
            1)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t1, 7000, one),
            2000)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t1, 9999, one),
            4999)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t1, 10001, one),
            5000)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t1, 14001, one),
            5000)

    def test_ffdbf_constrained(self):
        one = Fraction(1)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t2, 0, one),
            0)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t2, 1000, one),
            0)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t2, 2001, one),
            1)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t2, 4000, one),
            2000)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t2, 6999, one),
            4999)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t2, 10001, one),
            5000)
        self.assertEqual(
            gedf.ffdbf.ffdbf(self.t2, 12001, one),
            5001)



    def test_test_points(self):
        one = Fraction(1)
        pts = gedf.ffdbf.test_points(self.t1, one, 0)
        pts = iter(pts)
        self.assertEqual(pts.next(),  5000)
        self.assertEqual(pts.next(), 10000)
        self.assertEqual(pts.next(), 15000)
        self.assertEqual(pts.next(), 20000)


        pts = gedf.ffdbf.test_points(self.t2, one, 0)
        pts = iter(pts)
        self.assertEqual(pts.next(),  2000)
        self.assertEqual(pts.next(),  7000)
        self.assertEqual(pts.next(), 12000)
        self.assertEqual(pts.next(), 17000)


        pts = gedf.ffdbf.test_points(self.t1, Fraction(1, 2), 0)
        pts = iter(pts)
        self.assertEqual(pts.next(), 10000)
        self.assertEqual(pts.next(), 10000)
        self.assertEqual(pts.next(), 20000)

        pts = gedf.ffdbf.test_points(self.t2, Fraction(8, 10), 0)
        pts = iter(pts)
        self.assertEqual(pts.next(),   750)
        self.assertEqual(pts.next(),  7000)
        self.assertEqual(pts.next(), 10750)
        self.assertEqual(pts.next(), 17000)


    def test_testing_set(self):
        one = Fraction(1)
        ts = TaskSystem([self.t1, self.t2])
        pts = gedf.ffdbf.testing_set(ts, one, 0)
        ts = iter(pts)
        self.assertEqual(pts.next(),  2000)
        self.assertEqual(pts.next(),  5000)
        self.assertEqual(pts.next(),  7000)
        self.assertEqual(pts.next(), 10000)
        self.assertEqual(pts.next(), 12000)
        self.assertEqual(pts.next(), 15000)
        self.assertEqual(pts.next(), 17000)
        self.assertEqual(pts.next(), 20000)

# test test_points, with t_cur > 0 etc.

