import util
import unittest
import random

class TestMathHelpers(unittest.TestCase):

    def test_gcd(self):
        self.assertEqual(util.gcd(10, 20),
                         10)
        self.assertEqual(util.gcd(113, 17),
                         1)
        self.assertEqual(util.gcd(-23, 17),
                         1)
        self.assertEqual(util.gcd(-23, -54),
                         1)

    def test_lcm(self):
        self.assertEqual(util.lcm(10, 20),
                         20)
        self.assertEqual(util.lcm(3, 4),
                         12)
        self.assertEqual(util.lcm(10, 20, 3, 4),
                         60)

    def test_lcm_weird(self):
        self.assertEqual(util.lcm(),
                         0)
        self.assertEqual(util.lcm(10),
                         10)

class TestIterTools(unittest.TestCase):

    def setUp(self):
        self.times = 100

    def test_iskip(self):
        for x in xrange(self.times):
            n = random.randint(100, 1000)
            s = random.randint(0, 1100)
            self.assertEqual(range(1, n)[s:],
                             list(util.iskip(xrange(1, n), s)))

    def test_itimes(self):
        n = random.randint(100, 1000)
        self.assertEqual(list(util.itimes(xrange(1, n))),
                         list(xrange(1, n)))
        self.assertEqual(list(util.itimes(xrange(1, 5), f=lambda a: 4 - a)),
                         [1, 1, 1, 2, 2, 3])
        self.assertEqual(list(util.itimes(xrange(1, 10), f=lambda a: 3 - a)),
                         [1, 1, 2])

    def test_imerge(self):
        for x in xrange(self.times):
            a = range(10)
            b = range(1, 6)
            c = range(3, 14)
            a.reverse()
            b.reverse()
            c.reverse()
            self.assertEqual(list(util.imerge(lambda a,b: a >= b, a, b, c)),
                             [13, 12,11, 10,
                              9, 9, 8, 8, 7, 7, 6, 6,
                              5, 5, 5, 4, 4, 4, 3, 3, 3,
                              2, 2, 1, 1,
                              0])

    def test_neighbours(self):
        lst = list(util.neighbours(xrange(5)))
        self.assertEqual([(0, 1), (1, 2), (2, 3), (3, 4)], lst)

    def test_iremove(self):
        self.assertEqual(list(util.iremove(range(10) + range(55), xrange(5, 50))),
                         range(5) + range(10) + range(50, 55))


    def test_uniq_itimes(self):
        self.assertEqual(list(util.uniq(util.itimes(xrange(1, 1000), lambda i: i))),
                         range(1, 1000))

class Random(unittest.TestCase):

    def setUp(self):
        self.tmp_name = "/tmp/__test_prng_state.txt"

    def test_save_load_state(self):
        util.save_random_state(self.tmp_name)
        nums1 = [random.randint(0, 1000) for _ in xrange(100)]
        util.load_random_state(self.tmp_name)
        nums2 = [random.randint(0, 1000) for _ in xrange(100)]
        self.assertEqual(nums1, nums2)

if __name__ == '__main__':
    unittest.main()
