# I/O helpers---formatting, etc.

import sys
import random
import csv

from util.storage import storage

def print_row(row, f=sys.stdout, col_width=14, col_sep=" ",
              prepend='', append='\n'):
    field_fmt = "%%%ds" % col_width
    f.write(prepend)
    f.write(col_sep.join([field_fmt % str(field) for field in row]))
    f.write(append)
    f.flush()

def print_rows(rows, *args, **kargs):
    for r in rows:
        print_row(r, *args, **kargs)

def dotted(label, width=10):
    return label + '.' *  (width - len(label))

def header(label, width=78, fill="#"):
    n = len(label)
    l = (width - n) / 2
    r = width - n - l
    return fill * l + label + fill * r

def get_arg(index, default, list=sys.argv):
    "safely retrieve command line argument"
    if len(list) > index:
        return list[index]
    else:
        return default

def save_random_state(fname):
    f = open(fname, 'w')
    s = random.getstate()
    f.write(repr(s))
    f.close()

def load_random_state(fname):
    f = open(fname, 'r')
    s = eval(f.readline())
    f.close()
    random.setstate(s)

def load_csv(fname, convert=lambda x: x):
    f = open(fname)
    d = [[convert(x) for x in row]
         for row in csv.reader(f)]
    f.close()
    return d

def load_column_csv(fname,
                    convert=lambda x: x,
                    expect_uniform=True):
    f = open(fname)
    d = list(csv.reader(f))
    f.close()

    # infer column labels
    col_idx = {}
    for i, key in enumerate(d[0]):
        col_idx[key.strip()] = i

    max_idx = i

    data    = d[1:]

    if expect_uniform:
        for row in data:
            if len(row) != max_idx + 1:
                print len(row), max_idx
                msg = "expected uniform row length (%s:%d)" % (fname, data.index(row) + 1)
                raise IOError, msg # bad row length

    # column iterator
    def col(i):
        for row in data:
            if row:
                yield convert(row[i])

    by_col_name = {}
    by_col_idx  = [0] * (max_idx + 1)

    for key in col_idx:
        by_col_name[key] = list(col(col_idx[key]))
        by_col_idx[col_idx[key]] = by_col_name[key]

    return storage(name=fname, columns=col_idx,
                   by_name=by_col_name, by_idx=by_col_idx)
